package routing

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"orus.io/xbus/xbus-stdlib/service"
	"orus.io/xbus/xbus-stdlib/service/svctest"
	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/envelope"
)

func TestRouter(t *testing.T) {
	table := NewMemoryTableStore("test")

	tester := svctest.TestWorker(t, NewRouterAsWorker,
		&api.Actor{Name: "bob"},
		service.Settings{
			"store.backend":  "memory",
			"store.name":     "test",
			"extract-key-jq": ".ref",
		},
	)

	table.Set("FILE001", "out1")
	table.Set("FILE002", "out2")

	t.Run("envelope1", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE001"}`))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.NoError(t, err)

		assert.Equal(t, 1, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))

		if assert.Contains(t, outputs.Forwarded, "out1") {
			assert.Equal(t, e.ID(), outputs.Forwarded["out1"])
		}
	})

	t.Run("envelope2", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE002"}`))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.NoError(t, err)

		assert.Equal(t, 1, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))

		if assert.Contains(t, outputs.Forwarded, "out2") {
			assert.Equal(t, e.ID(), outputs.Forwarded["out2"])
		}
	})

	t.Run("missing entry", func(t *testing.T) {
		e := envelope.MustNewEnvelope(
			envelope.MustNewTextMessage("msg", `{"ref":"FILE003"}`))

		outputs, err := tester.Run(svctest.Inputs{api.DefaultInput: e})

		assert.Error(t, err)

		assert.Equal(t, 0, len(outputs.Forwarded))
		assert.Equal(t, 0, len(outputs.Sent))
	})
}
