package service

import (
	"context"
	"fmt"

	"xbus.io/go-xbus/v3"
	"xbus.io/go-xbus/v3/api"
	"xbus.io/go-xbus/v3/envelope"
)

type realInputs struct {
	ctx       context.Context
	client    *xbus.Client
	actorID   api.UUID
	actorKind string
	inputs    []*api.ActorProcessRequest_Input
}

func (ri realInputs) List() []string {
	names := make([]string, 0, len(ri.inputs))
	for _, input := range ri.inputs {
		names = append(names, input.Name)
	}
	return names
}

func (ri realInputs) Get(name string) (envelope.Envelope, error) {
	var input *api.ActorProcessRequest_Input
	for _, input = range ri.inputs {
		if input.Name == name {
			break
		}
	}
	if input.Name != name {
		return nil, fmt.Errorf("no such input: %s", name)
	}
	builder, err := ri.client.ReceiveEnvelope(
		ri.ctx, "actor", ri.actorID.String(), input.Envelope.ID, input.Envelope, input.Position)
	if err != nil {
		return nil, err
	}
	return builder.Envelope(), nil
}
