package service

import (
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestInvalidTypeError(t *testing.T) {
	assert.EqualError(t,
		InvalidTypeError{Key: "name", ExpectedType: "string", Value: 42},
		"invalid type for key 'name'. Expected a string, got a int: 42")
}

func TestSettings(t *testing.T) {
	t.Run("string", func(t *testing.T) {
		t.Run("get", func(t *testing.T) {
			value, err := Settings{"name": "bob"}.String("name")
			require.NoError(t, err)
			assert.Equal(t, "bob", value)
		})
		t.Run("default", func(t *testing.T) {
			value, err := Settings{}.StringD("name", "bob")
			require.NoError(t, err)
			assert.Equal(t, "bob", value)
		})
		t.Run("missing", func(t *testing.T) {
			value, err := Settings{}.String("name")
			require.IsType(t, NoSuchKeyError{}, err)
			assert.Equal(t, "name", err.(NoSuchKeyError).Key)
			assert.EqualError(t, err, "no such key: name")
			assert.Equal(t, "", value)
		})
		t.Run("invalid_type", func(t *testing.T) {
			value, err := Settings{"name": 42}.String("name")
			assert.Equal(t,
				InvalidTypeError{Key: "name", ExpectedType: "string", Value: 42},
				err)
			assert.Equal(t, "", value)
		})
	})
}
